//---------------------------------------------------------------------
// program:	nmc.c
//
// author:	Randy Hammock KC6HUR
//
// description:	This is a display module for use with the lcdproc
//		program. It is used to display certain information
//		about an IRLP node: The node number, the DTMF digit
//		that is currently being received, the last DTMF
//		command received, and up to 12 other oerating flags.
//
//		4-Line Display            2-Line Display
//		+--------------------+    +--------------------+
//              |NODE 4494   1 2 3 A |    |NODE 4494     CMD   |
//		| COMMAND    4 5 6 B |    |IEUTGBPC123         |
//		|            7 8 9 C |    +--------------------+
//		|IEUTGBPC123 * 0 # D |
//		+--------------------+
//
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include "shared/sockets.h"
#include "shared/debug.h"

#include "main.h"
#include "mode.h"
#include "nmc.h"
#include "irlpdev.h"

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif

#ifdef HAVE_SYS_CPUVAR_H
#include <sys/cpuvar.h>
#endif

#ifdef HAVE_LIBKSTAT
#include <kstat.h>
#endif

#if HAVE_ERRNO_H
#include <errno.h>
#endif

#if HAVE_LIMITS_H
#include <limits.h>
#endif

#if HAVE_KVM_H
#include <kvm.h>
#endif

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#if HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif

#if HAVE_SYS_SYSCTL_H
#include <sys/sysctl.h>
#endif

#if HAVE_SYS_SCHED_H
#include <sys/sched.h>
#endif

#if HAVE_SYS_DKSTAT_H
#include <sys/dkstat.h>
#endif

// IRLP Node Status Flags
#define IRLP 0x0001
#define ECHO 0x0002
#define USER 0x0004
#define TIMR 0x0008
#define GATE 0x0100
#define BUSY 0x0200

char *node;

int nmc_init () {
    node = getenv("STATIONID");
    // Open the /dev/irlp-port device
    // Channel to the IRLP hardware
    if (irlpdev_open() < 0) {
        perror("Can't access parallel port");
        return errno;
    }
    return 0;
}

int nmc_close () {

    return 0;
}

////////////////////////////////////////////////////////////////////////
// IRLP nmc screen
//
int nmc_screen (int rep, int display) {
#undef NMC_BUF_SIZE
#define NMC_BUF_SIZE 4
    FILE *file;
//    int i, j, n;
    static int dtmf;
    unsigned char c[2];			// Returned string from parallel port
    int status = 0;			// IRLP Port Status
    int nodestat = 0;			// IRLP Node Status
    static int stath = 0;		// IRLP Port Status History
    static int nodeh = 0;		// IRLP Node Status History
    static int first = 1;

    char *dtmf_str = " 1234567890*#ABCD";

    if (first) {
        first = 0;
        //-------------------------------------------------------------
        // Initialize the display (do it only once)
        //-------------------------------------------------------------
        sock_send_string (sock, "screen_add N\n");
        sprintf (buffer, "screen_set N -name {NMC: %s}\n", host);
        sock_send_string (sock, buffer);
        if (lcd_hgt >= 4) {
            // Define the 4-line display widgets
            sock_send_string (sock, "widget_add N line1 string\n");
            sock_send_string (sock, "widget_add N line2 string\n");
            sock_send_string (sock, "widget_add N line3 string\n");
            sock_send_string (sock, "widget_add N line4 string\n");
            sock_send_string (sock, "widget_add N dtmf1 string\n");
            sock_send_string (sock, "widget_add N dtmf2 string\n");
            sock_send_string (sock, "widget_add N dtmf3 string\n");
            sock_send_string (sock, "widget_add N dtmf4 string\n");
            // Set the initial widget values
            sprintf (buffer, "widget_set N line1 1 1 {NODE %s  }\n", node+3);
            sock_send_string (sock, buffer);
            sock_send_string (sock, "widget_set N line2 1 2 { COMMAND   }\n");
            sock_send_string (sock, "widget_set N line3 1 3 {           }\n");
            sock_send_string (sock, "widget_set N line4 1 4 {           }\n");
            sock_send_string (sock, "widget_set N dtmf1 12 1 { 1 2 3 A }\n");
            sock_send_string (sock, "widget_set N dtmf2 12 2 { 4 5 6 B }\n");
            sock_send_string (sock, "widget_set N dtmf3 12 3 { 7 8 9 C }\n");
            sock_send_string (sock, "widget_set N dtmf4 12 4 { * 0 # D }\n");
        } else {
            // Define the 2-line display widget
            sock_send_string (sock, "widget_add N line1 string\n");
            sock_send_string (sock, "widget_add N line2 string\n");
            // Set the inital widget values
            sprintf (buffer, "widget_set N line1 1 1 {NODE %s     CMD   }\n", node+3);
            sock_send_string (sock, buffer);
            sock_send_string (sock, "widget_set N line2 1 2 {           }\n");
        }
    }

    //-----------------------------------------------------------------
    // Read IRLP Hardware Status
    //-----------------------------------------------------------------
    read_irlpdev(c, 2);				// Read parallel port
    if ((file=fopen("/home/irlp/local/enable", "r")) != NULL) {
        nodestat |= IRLP;
        fclose(file);
    }
    if ((file=fopen("/home/irlp/local/echo_enable", "r")) != NULL) {
        nodestat |= ECHO;
        fclose(file);
    }
    if ((file=fopen("/home/irlp/local/nouser", "r")) == NULL) {
        nodestat |= USER;
    } else {
        fclose(file);
    }
    if ((file=fopen("/home/irlp/local/timeout", "r")) != NULL) {
        nodestat |= TIMR;
        fclose(file);
    }
    if ((file=fopen("/home/irlp/local/preaccess","r")) != NULL) {
        nodestat |= GATE;
        fclose(file);
    }
    if ((file=fopen("/home/irlp/local/cos_active", "r")) != NULL) {
        nodestat |= BUSY;
        fclose(file);
    }
    status = (int)(c[1] << 8) | (int)(c[0] & 0xff);
    if ((status == stath) && (nodestat == nodeh))// If status unchaged
        return 0;				// no processing needed
    stath = status;				// record new status
    nodeh = nodestat;				// record new node status
    dtmf = ((status >> 3) & 0x000f);		// Extract DTMD digit

    //-----------------------------------------------------------------
    // Display current IRLP status flags (both 2 and 4 line displays)
    //-----------------------------------------------------------------
    sprintf (buffer, "widget_set N %s {%c%c%c%c%c%c%c%c%c%c%c}\n",
        (lcd_hgt >= 4)?"line4 1 4":"line2 1 2",	// 2 or 4 line selection
        (nodestat & IRLP)?'I':' ',              // IRLP flag
        (nodestat & ECHO)?'E':' ',              // ECHO flag
        (nodestat & USER)?'U':' ',              // USER flag
        (nodestat & TIMR)?'T':' ',              // TIMR flag
        (nodestat & GATE)?'G':' ',              // GATE flag
        (nodestat & BUSY)?'B':' ',              // BUSY flag
        (status & 0x0200)?((status & 0x8000)?'F':'P'):' ',// PTT flag
        (status & 0x0080)?'C':' ',		// COS flag
        (status & 0x0400)?'1':' ',		// AUX1 flag
        (status & 0x0800)?'2':' ',		// AUX2 flag
        (status & 0x1000)?'3':' ');		// AUX3 flag
    sock_send_string (sock, buffer);		// Display status flags

    if (lcd_hgt >= 4) {
        // Add 4 line display processing here
        //-------------------------------------------------------------
        // Indicate current DTMF digit being received
        //-------------------------------------------------------------
        sprintf (buffer, "widget_set N dtmf1 12 1 { %c %c %c %c }\n",
            (dtmf == 0x0001)?' ':'1',
            (dtmf == 0x0002)?' ':'2',
            (dtmf == 0x0003)?' ':'3',
            (dtmf == 0x000d)?' ':'A');
        sock_send_string (sock, buffer);
        sprintf (buffer, "widget_set N dtmf2 12 2 { %c %c %c %c }\n",
            (dtmf == 0x0004)?' ':'4',
            (dtmf == 0x0005)?' ':'5',
            (dtmf == 0x0006)?' ':'6',
            (dtmf == 0x000e)?' ':'B');
        sock_send_string (sock, buffer);
        sprintf (buffer, "widget_set N dtmf3 12 3 { %c %c %c %c }\n",
            (dtmf == 0x0007)?' ':'7',
            (dtmf == 0x0008)?' ':'8',
            (dtmf == 0x0009)?' ':'9',
            (dtmf == 0x000f)?' ':'C');
        sock_send_string (sock, buffer);
        // NOTE: IRLP does not decode the "D" digit.
        sprintf (buffer, "widget_set N dtmf4 12 4 { %c %c %c D }\n",
            (dtmf == 0x000b)?' ':'*',
            (dtmf == 0x000a)?' ':'0',
            (dtmf == 0x000c)?' ':'#');
        sock_send_string (sock, buffer);

    } else {
        // Add 2 line display processing here
        sprintf (buffer, "widget_set N line1 1 1 {NODE %s  %c  CMD   }\n",
            node+3,
            dtmf_str[dtmf]);
        sock_send_string (sock, buffer);
    }

    return 0;
}		// End nmc_screen()

